@@ -3,6 +3,8 @@ from typing import List, Any, Tuple, Callable, Dict, Union, Iterable
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
+import stumpy
+from matplotlib.patches import Rectangle
 from .config import Style
 from .utils import PickleUtils, Rosetta, make_dict, get_num_workers, process_queue
 from .data_quality import basis_quality_checks, basis_quality_plots, fill_ratio
@@ -14,6 +16,7 @@ import pathlib
 from pydantic import BaseModel
 from sklearn.feature_selection import SelectKBest, chi2
 from tqdm.auto import tqdm
+import matrixprofile
 
 
 class SVD(BaseModel):
@@ -356,6 +359,68 @@ class VectorMultiset(PickleUtils, Style, Rosetta):
             to_return.drop_col_types(drop_types=drop_types, inplace=True)
             return to_return
 
+    def matrix_profile_univariate(self, col_names: Union[str, Iterable[str]],
+                                  **kwargs) -> List[Tuple[Any, List[plt.Figure]]]:
+        """
+        Very thin wrapper around matrixprofile
+
+        :param col_names: a column name (or list of column names) to be profiled
+        :param kwargs: keyword arguments passed through to matrixprofile.analyze()
+        :return: List of outputs from matrixprofile.analyze()
+        """
+        queue: List[str] = [col_names] if isinstance(col_names, str) else col_names
+        return [matrixprofile.analyze(self.data.loc[:, q].tolist(), **kwargs) for q in queue]
+
+    def calculate_stumpy_profile_univariate(self, col_name: str, window_size: int, **kwargs) -> np.ndarray:
+        """
+        Very thin wrapper around stumpy's matrix profile method. NB: for use in MultiSet, user is responsible for order
+
+        :param col_name: column name to profile
+        :param window_size: sliding window size
+        :param kwargs: keyword arguments for stumpy.stump()
+        :return: numpy array with the profile
+        """
+        return stumpy.stump(self.data.loc[:, col_name].tolist(), window_size, **kwargs)
+
+    def stumpy_profile_univariate(self, col_name: str, window_size: int, annotate_motif: bool = True,
+                                  **kwargs) -> plt.Figure:
+        """
+        Plot of data & matrix profile (with optional annotation). NB: for use in MultiSet, user is responsible for order
+
+        :param col_name: name of the column to plot
+        :param window_size: sliding window size
+        :param annotate_motif: whether to highlight the main motif and its nearest neighbor
+        :param kwargs: keyword arguments for plots and stumpy.stump()
+        :return: figure handle of the plot
+        """
+        # Make the plot and profile
+        figsize: Any = kwargs.pop('figsize', self.figsize)  # typically a 2-element tuple or list
+        title: str = kwargs.pop('title', self.translate(self.name_root))
+        profile: np.ndarray = self.calculate_stumpy_profile_univariate(col_name, window_size, **kwargs)
+        minimum: float = float(np.nanmin(self.data.loc[:, col_name].tolist()))
+        maximum: float = float(np.nanmax(self.data.loc[:, col_name].tolist()))
+        fig, axs = plt.subplots(2, sharex='all', gridspec_kw={'hspace': 0},
+                                figsize=figsize, facecolor=self.facecolor)
+        plt.suptitle(title, fontsize=self.title_fontsize)
+        axs[0].plot(self.data.loc[:, col_name].tolist(), color=kwargs.get('color', self.color))
+        axs[0].set_ylabel(self.translate(col_name), fontsize=self.label_fontsize)
+        axs[1].set_xlabel(self.translate('Index'), fontsize=self.label_fontsize)
+        axs[1].set_ylabel('Matrix Profile', fontsize=self.label_fontsize)
+        axs[1].plot(profile[:, 0], color=kwargs.get('color', self.color))
+
+        # Annotate the main motif (and its nearest neighbor) if desired
+        if annotate_motif:
+            motif_idx = np.argsort(profile[:, 0])[0]
+            nearest_neighbor_idx = profile[motif_idx, 1]
+            rect = Rectangle((motif_idx, minimum), window_size, maximum, facecolor='palegoldenrod')
+            axs[0].add_patch(rect)
+            rect = Rectangle((nearest_neighbor_idx, minimum), window_size, maximum, facecolor='palegoldenrod')
+            axs[0].add_patch(rect)
+            axs[1].axvline(x=motif_idx, linestyle="dashed", color='k')
+            axs[1].axvline(x=nearest_neighbor_idx, linestyle="dashed", color='k')
+
+        return fig
+
 
 class SlidingWindowResults(VectorMultiset):
     """ Results from Sequence.sliding_window(), which is a VectorMultiset with extra context baked in """
@@ -442,6 +507,7 @@ class VectorSequence(VectorMultiset):
     # (Inherits `data` dataframe and `name_root` string from VectorMultiset)
     basis_col_name: str = 'basis'
     error_if_basis_quality_issues: bool = False
+    info_surface: InfoSurface = None
 
     def __init__(self, skip_vector_sequence_init: bool = False, **kwargs: Any):
         super().__init__(**kwargs)
@@ -789,26 +855,32 @@ class VectorSequence(VectorMultiset):
         return svd
 
     def calculate_info_surface(self, window_widths: List[float] = None, cols: Union[str, List[str]] = None, *args,
-                               **kwargs) -> InfoSurface:
+                               save_result: bool = True, **kwargs) -> InfoSurface:
         """ Calculates the info surface by sliding the SVD function along the basis
 
         :param window_widths: window widths for the sliding window analysis
         :param cols: which data features to use in the svd
         :param args: args for sliding window analysis and eval function
+        :param save_result: whether to save attach the infosurface to the object once calculated
         :param kwargs: kwargs for sliding window analysis and eval function
         :return: InfoSurface object
         """
         result: SlidingWindowResults = self.sliding_window(info_surface_slider, window_widths=window_widths, cols=cols,
                                                            *args, **kwargs)
-        return InfoSurface(**result.dict())
+        info_surface: InfoSurface = InfoSurface(**result.dict())
+        if save_result:
+            self.info_surface = info_surface
+        return info_surface
 
     def plot_info_surface(self, window_widths: List[float] = None, cols: Union[str, List[str]] = None,
-                          singular_values: List[int] = None, style: Style = None, *args, **kwargs) -> List[plt.Figure]:
+                          singular_values: List[int] = None, style: Style = None, use_cache: bool = False,
+                          *args, **kwargs) -> List[plt.Figure]:
         """ Calculates and plots the info surface: value of singular vectors as a function of window start and width
 
         :param window_widths: window widths for the sliding window analysis
         :param cols: which data features to use in the svd
         :param singular_values: singular values for which to plot the surface (default: [1, 2, 3])
+        :param use_cache: uses cached info surface if available, otherwise calculates a fresh one
         :param args: args for sliding window analysis and eval function
         :param kwargs: kwargs for sliding window analysis and eval function
         :param style: isthmuslib Style object for the colormap
@@ -820,9 +892,13 @@ class VectorSequence(VectorMultiset):
         svd_kwargs: Dict[str, Any] = {k: v for k, v in kwargs.items() if k not in config.dict()}
         style_kwargs: Dict[str, Any] = {k: v for k, v in kwargs.items() if k in config.dict()}
 
-        result: InfoSurface = self.calculate_info_surface(window_widths=window_widths, cols=cols, *args, **svd_kwargs)
-        return result.plot_info_surface(singular_values=singular_values,
-                                        **{**make_dict(style), **make_dict(style_kwargs)})
+        if use_cache and self.info_surface:
+            info_surface: InfoSurface = self.info_surface
+        else:
+            info_surface: InfoSurface = self.calculate_info_surface(window_widths=window_widths, cols=cols,
+                                                                    *args, **svd_kwargs)
+        return info_surface.plot_info_surface(singular_values=singular_values,
+                                              **{**make_dict(style), **make_dict(style_kwargs)})
 
     def correlation_matrix(self, exclude_basis: bool = True, **kwargs) -> pd.DataFrame:
         """
